#include "connectserverdialog.h"
#include <QMessageBox>
#include <QPushButton>
#include <settings.h>
#include "application.h"
#include <QListWidget>
#include <QString>

namespace NDEFileMAN {

ConnectServerDialog::ConnectServerDialog(QWidget *parent): QDialog(parent) {
      serverTypes = QList<ServerType>{
        {tr("SSH"), "sftp", 22, false},
        {tr("FTP"), "ftp", 21, true},
        {tr("WebDav"), "dav", 80, true},
        {tr("Secure WebDav"), "davs", 443, false},
        {tr("HTTP"), "http", 80, true},
        {tr("HTTPS"), "https", 443, true},
        {tr("Window Share"), "smb",  139, true}
      };

      ui.setupUi(this);

      ui.buttonBox->button(QDialogButtonBox::Ok)->setText(tr("OK"));
      ui.buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("Cancel"));

      connect(ui.serverType, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
              this, &ConnectServerDialog::onCurrentIndexChanged);

      connect(ui.host, &QComboBox::editTextChanged, this, &ConnectServerDialog::checkInput);
      connect(ui.host, static_cast<void(QComboBox::*)(int)>(&QComboBox::activated),
              this, &ConnectServerDialog::onCurrentIndexChanged);

      connect(ui.userName, &QLineEdit::textChanged, this, &ConnectServerDialog::checkInput);
      for(const auto& serverType : const_cast<const QList<ServerType>&>(serverTypes)) {
        ui.serverType->addItem(serverType.name);
      }

      ui.serverType->setCurrentIndex(0);
      //onCurrentIndexChanged(0);

      connectInfo = new connectFavHisData;

      connectFavoriteData = connectInfo->loadData(connectFavHisData::Favorites);
      for (int i = 0; i < connectFavoriteData.count(); i++)
      {
          QListWidgetItem *item = new QListWidgetItem(ui.listWidget_fav);
          item->setFlags(item->flags() | Qt::ItemIsEditable);
          if (connectFavoriteData.at(i).favoriveName == "")
            item->setText(connectFavoriteData.at(i).host + "  " + connectFavoriteData.at(i).userName + "  " + connectFavoriteData.at(i).scheme);
          else
            item->setText(connectFavoriteData.at(i).favoriveName);
      }
      ui.listWidget_fav->setEditTriggers(QAbstractItemView::SelectedClicked);
      connectHistoryData = connectInfo->loadData(connectFavHisData::Historys);
      for (int i = 0; i < connectHistoryData.count(); i++)
      {
          ui.host->addItem(connectHistoryData.at(i).host + "  " + connectHistoryData.at(i).userName + "  " + connectHistoryData.at(i).scheme);
      }
      if (connectHistoryData.count() != 0)
        ui.host->addItem(tr("Clear all historys"));

      ui.host->setCurrentText("");

      QObject::connect(ui.listWidget_fav, SIGNAL(itemDoubleClicked(QListWidgetItem *)), this, SLOT(listItemClick(QListWidgetItem *)));
      QObject::connect(ui.listWidget_fav, SIGNAL(itemChanged(QListWidgetItem *)), this, SLOT(favoriteTextChanged(QListWidgetItem *)));

      QObject::connect(ui.button_delete, SIGNAL(clicked()), this, SLOT(deleteFavorite()));

      QObject::disconnect(ui.buttonBox, SIGNAL(accepted()), this, SLOT(accepted()));
      QObject::connect(ui.buttonBox, SIGNAL(accepted()), this, SLOT(buttonOkClick()));
}

ConnectServerDialog::~ConnectServerDialog() {
    delete connectInfo;
}

void ConnectServerDialog::deleteFavorite()
{
    int index = ui.listWidget_fav->currentRow();
    if (index < 0)
        return ;

    connectInfo->deleteData(index, connectFavHisData::Favorites);
    ui.listWidget_fav->takeItem(index);
    connectFavoriteData.removeAt(index);
}

void ConnectServerDialog::listItemClick(QListWidgetItem *item)
{
    int index = ui.listWidget_fav->row(item);
    ui.host->setCurrentText(connectFavoriteData.at(index).host);
    ui.serverType->setCurrentText(connectFavoriteData.at(index).scheme);
    ui.port->setValue(connectFavoriteData.at(index).port.toInt());
    ui.path->setText(connectFavoriteData.at(index).path);
    if (connectFavoriteData.at(index).canAnonymous == "no")
    {
        ui.loginAsUser->setChecked(true);
        ui.userName->setText(connectFavoriteData.at(index).userName);
    }
    else
        ui.ananymousLogin->setChecked(true);

}

void ConnectServerDialog::favoriteTextChanged(QListWidgetItem *item)
{
    int index = ui.listWidget_fav->row(item);
    connectInfo->changeFavoriteName(index, item->text());
}

void ConnectServerDialog::buttonOkClick()
{
    connectFavHisData::data temp;
    temp.host = ui.host->currentText();
    temp.scheme = ui.serverType->currentText();
    temp.port = QString::number(ui.port->value());
    temp.path = ui.path->text();
    if (ui.loginAsUser->isChecked())
        temp.canAnonymous = "no";
    else
        temp.canAnonymous = "yes";
    temp.userName = ui.userName->text();

    if (ui.checkBox_fav->isChecked()){
        connectInfo->addData(temp,connectFavHisData::Favorites);
    }

    connectInfo->addData(temp,connectFavHisData::Historys);

}

QString ConnectServerDialog::uriText() {
      QString uri;
      int serverTypeIdx = ui.serverType->currentIndex();
      const auto& serverType = serverTypes[serverTypeIdx];

      // make an URI from the data
      uri = QString::fromLatin1(serverType.scheme);
      uri += QStringLiteral("://");
      if(ui.loginAsUser->isChecked()) {
        uri += ui.userName->text();
        uri += '@';
      }

      uri += ui.host->currentText();
      int port = ui.port->value();
      if(port != serverType.defaultPort) {
        uri += ':';
        uri += QString::number(port);
      }

      QString path = ui.path->text();
      if(path.isEmpty() || path[0] != '/') {
        uri += '/';
      }
      uri += path;
      return uri;
}

void ConnectServerDialog::onCurrentIndexChanged(int /*index*/) {
    QComboBox* box = static_cast<QComboBox*>(sender());

    if (box == ui.serverType)
    {
        ui.loginAsUser->setEnabled(true);
        ui.port->setEnabled(true);
        ui.path->setEnabled(true);

        int serverTypeIdx = ui.serverType->currentIndex();
        const auto& serverType = serverTypes[serverTypeIdx];
        ui.port->setValue(serverType.defaultPort);
        ui.ananymousLogin->setEnabled(serverType.canAnonymous);

        if(serverType.canAnonymous){
            ui.ananymousLogin->setChecked(true);
        }
        else
            ui.loginAsUser->setChecked(true);

          ui.host->setFocus();
          checkInput();
    }else if(box == ui.host){
        int index = ui.host->currentIndex();
        if (index == connectHistoryData.count())
        {
            ui.host->clear();
            connectInfo->clearHistory();
            return;
        }
        ui.host->setCurrentText(connectHistoryData.at(index).host);
        ui.serverType->setCurrentText(connectHistoryData.at(index).scheme);
        ui.port->setValue(connectHistoryData.at(index).port.toInt());
        ui.path->setText(connectHistoryData.at(index).path);
        if (connectHistoryData.at(index).canAnonymous == "no")
        {
            ui.loginAsUser->setChecked(true);
            ui.userName->setText(connectHistoryData.at(index).userName);
        }
        else
            ui.ananymousLogin->setChecked(true);
    }
    if (ui.ananymousLogin->isChecked())
        ui.userName->clear();

    if (ui.serverType->currentText() == tr("Window Share"))
    {
        ui.path->clear();
        ui.loginAsUser->setEnabled(false);
        ui.port->setEnabled(false);
        ui.path->setEnabled(false);
    }

}

void ConnectServerDialog::checkInput() {
      bool valid = true;
      if(ui.host->currentText().isEmpty()) {
        valid = false;
      }
      else if(ui.loginAsUser->isChecked() && ui.userName->text().isEmpty()) {
        valid = false;
      }
      ui.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(valid);
}

} // namespace NDEFileMAN
