/*

    Copyright (C) 2013  Hong Jen Yee (PCMan) <pcman.tw@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


#ifndef NDEFILEMAN_FOLDERVIEW_H
#define NDEFILEMAN_FOLDERVIEW_H

#include <libfm-qt/folderview.h>
#include <libfm-qt/core/filepath.h>


namespace Fm {
class FileMenu;
class FolderMenu;
}

namespace NDEFileMAN {

class Settings;

class View : public Fm::FolderView {
    Q_OBJECT
public:
    enum DirectoryView {
            IconView,
            CompactView,
            DetailedList,
            ThumbnailView
        };

    explicit View(Fm::FolderView::ViewMode _mode = IconMode, QWidget* parent = 0);
    virtual ~View();

    void updateFromSettings(Settings& settings);

    QSize  getMargins() const {
        return Fm::FolderView::getMargins();
    }
    void setMargins(QSize size) {
        Fm::FolderView::setMargins(size);
    }

    void setDesktopPrepare(bool type = false) {
        isDesktopPrepare_ = type;
    }
    void setFolderViewMode(Fm::FolderView::ViewMode mode);
    bool getFolderReload() {
        return isFolderReload_;
    }
    void setFolderReload(bool reload) {
        isFolderReload_ = reload;
    }

Q_SIGNALS:
    void openDirRequested(const Fm::FilePath& path, int target);
    void folderViewMode(Fm::FolderView::ViewMode mode);

protected Q_SLOTS:
    void onNewWindow();
    void onNewTab();
    void onOpenInTerminal();
    void onSearch();
    void onCreateShortcutWithTriggered(); // 右键图标创建快捷方式
    void onOpenFolderTerminal();          // 右键菜单打开终端
    void onOpenAsRootTriggered();         // 以管理员权限运行应用
    void onCopyToHomeTriggered();         // 发送到主文件夹
    void onCopyToDesktopTriggered();      // 发送到桌面
    void onFolderReload();                // 刷新
    void DirectoryViewSlots(int mode);    // 文件管理器查看功能信号接收
    void onDesktopShortcutTriggered();    // 发送到桌面快捷方式

protected:
    virtual void onFileClicked(int type, const std::shared_ptr<const Fm::FileInfo>& fileInfo);
    virtual void prepareFileMenu(Fm::FileMenu* menu);
    virtual void prepareFolderMenu(Fm::FolderMenu* menu);

private:
    QMenu* createCopytoMenuSub(Fm::FileMenu* menu);
    bool isDesktopPrepare_;
    bool isFolderReload_;
};

}
#endif // NDEFILEMAN_FOLDERVIEW_H
